clc;
clear all;

%%% CVX toolbox setup
  cvx_setup 

%Data Reading from Text file
C = load('steady_states.txt', '-ascii');
x1s = C(:,1); x2s = C(:,2); 
% three steady states are x1s,x2s,x3s. Desired and initial steady states will be any one of these two states.
%x1s== x_{I}^{s2}
%x2s=x_{I}^{s1}
%parameter reading
k = load('parameter.txt', '-ascii');
S = load('StoichiometricMatrix.txt', '-ascii');
    
xis=x2s;    % Initial Steady-state
xds=x1s;    %Desired Steady-state
[n m]=size(S);   %n= #rows of Stoichiometric matrix= number of stattes  and m= #column
b1=xds-xis;
S1=S*diag(k);
 R=eye(n);
 W=eye(m);
 w1=1000;   %Weight of beta 
 w2=0.001;  %Weight of Deltax and alpha
variableStateIndx = [1,2,5,7,10,15,18];
fixedStateIndx = [3,4,6,8,9,11,12,13,14,16,17,19,20,21,22];
M = zeros(n-length(variableStateIndx),n);
for i=1:length(fixedStateIndx)
    M(i,fixedStateIndx(i)) = true;
end
 % Moieties   
conservedMoietyIndex{1}=[1,3,4,6,8,12];
conservedMoietyIndex{2}=[7,8,9,11,12,13,14,16,20];
conservedMoietyIndex{3}=[15,16,17,19,20,21,22];
conservedMoietyIndex{4}=[5,6];
conservedMoietyIndex{5}=[10,11,14];
conservedMoietyIndex{6}=[18,19,22];
conservedMoietyIndex{7}=[2,3];

numconservedMoiety=size(conservedMoietyIndex,2);
for i=1:numconservedMoiety
    a(i,conservedMoietyIndex{i})=true;
end


  Moietymin=[6*10^-4;0.24;0.24;6*10^-5;6*10^-5;2*10^-2;6*10^-5];
Moietymax=[1.5*10^-2;6;6;1.5*10^-3;1.5*10^-3;0.6;1.5*10^-3];


cvx_begin sdp 
 cvx_precision high
variables deltax(n) 
variables aalpha(m) positive
variables bbeta

xzs=xis+deltax+S1*aalpha; %%Positivity constraints

minimize (w1*bbeta+w2*deltax'*R*deltax+w2*aalpha'*W*aalpha)

subject to
[-bbeta,  b1'-deltax'-aalpha'*S1'; b1-deltax-S1*aalpha, -eye(22)]<= 0;
aalpha >= 0;
xis+deltax>=0.00000001; % positivity constraints
M*deltax ==0;           %fixed states
bbeta >= 10^-11;
xzs>=0;
Moietymax >= a*(xis+deltax) >= Moietymin ;
cvx_end
cvx_status
 cvx_optval
 bbeta
x=xis+deltax; %%% Input to the Simulink Model

sim('mapk_1st');
simulink_states1=[x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22];
for i=1:22
    finalstates(i,1)=simulink_states1(i).signals.values(end);
    i=i+1;
end
error= norm(finalstates-xds)/norm(xds);
normalizedErrorPercent=error*100;
%
%
% For generating of trajectories plot 
% x=xds;
% sim('mapk_1st')
% simulink_states2=[x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21,x22];
% 
% plotStyle = {'k','--k'};
% i=1;
% for p=[6,13,15,21]  % Index of states to be plotted
%     plot(simulink_states1(p).time, simulink_states1(p).signals.values,'k','LineWidth',3,'MarkerSize',2);     
%      xlabel('Time','FontSize',16);
%      ylabel('Concentration (\muM)','FontSize',16);
%      xt = get(gca, 'XTick');
%      set(gca, 'FontSize', 16)
%        hold on
%         plot(simulink_states2(p).time, simulink_states2(p).signals.values,'--k','LineWidth',3,'MarkerSize',2);     
%      xlabel('Time','FontSize',16);
%      ylabel('Concentration (\muM)','FontSize',16);
%      xt = get(gca, 'XTick');
%      set(gca, 'FontSize', 16)
%      hold on
% plot(NaN,'or')
% legend('Final state: x_{os}','Target state: x_{ds}=','x_{is}(13)+\Deltax(13)=','Location','best')
%  i=i+1;
%      figure(i);
% end
 